from typing_extensions import Annotated

from cookiecutter.main import cookiecutter
import typer

from tigr81 import LOCAL_REPO_LOCATION, REPO_LOCATION
from tigr81.cli_settings import CLI_SETTINGS
import tigr81.commands.core.scaffold as scaffold_core
from tigr81.commands.scaffold.project_template import (
    Dependency,
    ProjectTemplate,
    ProjectTemplateOptions,
    ProjectTypeEnum,
)

import pathlib as pl


def scaffold(
    project_type: Annotated[
        ProjectTypeEnum, typer.Argument(help="The project type to scaffold")
    ],
    default: bool = typer.Option(
        False, help="Set to False to enable input during cookiecutter execution"
    ),
    output_dir: pl.Path = typer.Option(
        pl.Path("."),
        help="Set if you want to scaffold the project template in a specific directory",
    ),
):
    """Scaffold a project template"""

    if project_type == ProjectTypeEnum.PRIME_REACT:
        scaffold_core.scaffold_cookiecutter(
            project_type=project_type,
            output_dir=output_dir
        )
    else:
        scaffold_core.scaffold(project_type=project_type, default=default, output_dir=output_dir)


if __name__ == "__main__":
    project_type = "fastapi"
    PROJECT_TEMPLATE_LOCATION = REPO_LOCATION
    checkout = "develop"

    if CLI_SETTINGS.tigr81_environment == "local":
        PROJECT_TEMPLATE_LOCATION = LOCAL_REPO_LOCATION.as_posix()
        checkout = None

    typer.echo(
        f"Scaffolding a {project_type} project template from {PROJECT_TEMPLATE_LOCATION}"
    )

    project_template = ProjectTemplate(
        project_type=project_type,
        project_options=ProjectTemplateOptions(
            name=project_type,
            package_name=project_type,
            description=project_type,
        ),
        dependencies=[
            Dependency(name="dagprep", relative_path="../../dagprep"),
            Dependency(name="fresko", relative_path="../../fresko"),
        ],
    )

    print(project_template.extra_content)

    cookiecutter(
        template=PROJECT_TEMPLATE_LOCATION,
        output_dir=".",
        no_input=True,
        # extra_context=project_template.project_options.model_dump(),
        extra_context=project_template.extra_content,
        checkout=checkout,
        directory=f"project_templates/{project_template.project_type}",
    )



import pathlib as pl

__version__ = "1.1.1"

PACKAGE_LOCATION = pl.Path(__file__).parent.resolve()
ROOT_LOCATION = PACKAGE_LOCATION.parent
LOCAL_REPO_LOCATION = ROOT_LOCATION.parent

LOCAL_PROJECT_TEMPLATES_FOLDER_LOCATION = LOCAL_REPO_LOCATION / "project_templates"
RESOURCE_LOCATION = PACKAGE_LOCATION / "resources"


PYPY_URL = "https://pypi.org/project/tigr81/"
REPO_LOCATION = "https://github.com/giuseppeambrosio97/tigr81.git"



import typer
from tigr81.commands.scaffold import scaffold
from tigr81.commands.monorepo import monorepo
from tigr81 import PYPY_URL, REPO_LOCATION, __version__ 

app = typer.Typer()

    
@app.command()
def version():
    typer.echo(f"v{__version__}")
    typer.echo("\nCheck out for new versions:")
    typer.echo(f"- {PYPY_URL}")
    typer.echo(f"- {REPO_LOCATION}")
    

app.command()(scaffold.scaffold)
app.add_typer(monorepo.app, name="monorepo")



from typing import List


def pretty_list(lst: List[str]) -> str:
    if not lst:
        return "Empty"
    
    if len(lst) == 1:
        return lst[0]

    return f'{", ".join(lst[:-1])} and {lst[-1]}' 
    



import subprocess as sp
import sys

import typer
import shutil
import pathlib as pl


class PoetryPM:
    """
    A utility class for managing Python dependencies using Poetry.

    This class provides methods to interact with the Poetry package manager.
    It checks for the presence of Poetry, installs components, and removes
    dependencies from a specified working directory.
    """

    def __init__(self):
        self.poetry_executable = shutil.which("poetry")
        if self.poetry_executable:
            result = sp.run(
                [self.poetry_executable, "--version"], 
                stdout=sp.PIPE, 
                stderr=sp.PIPE, 
                check=True, 
                text=True
            )
            poetry_version = result.stdout.strip()
            typer.echo(f"Poetry executable: {self.poetry_executable}")
            typer.echo(f"Poetry version: {poetry_version}")
        else:
            typer.echo("Poetry is not installed or not found on this machine.")
            sys.exit(1)

    def install(self, cwd: pl.Path) -> None:
        try:
            typer.echo(f"Installing component {cwd}...")
            result: sp.CompletedProcess = sp.run(
                [self.poetry_executable, "install"], 
                stdout=sp.PIPE, 
                stderr=sp.PIPE, 
                check=True, 
                cwd=cwd
            )

            typer.echo(result.stdout, color="green")
            typer.echo(result.stderr, color="red")

            if result.returncode != 0:
                typer.echo("An error occurred while running 'poetry install'.", color="red")
                sys.exit(1)
        except sp.CalledProcessError as e:
            typer.echo("An error occurred while running 'poetry install'.", color="red")
            sys.exit(1)

    def remove(self, cwd: pl.Path, dependency: str) -> None:
        try:
            typer.echo(f"Removing dependency {dependency} from {cwd}...")
            result: sp.CompletedProcess = sp.run(
                [self.poetry_executable, "remove", dependency], 
                stdout=sp.PIPE, 
                stderr=sp.PIPE, 
                check=True, 
                cwd=cwd
            )

            typer.echo(result.stdout, color="green")
            typer.echo(result.stderr, color="red")

            if result.returncode != 0:
                typer.echo(f"An error occurred while running 'poetry remove {dependency}'.", color="red")
                sys.exit(1)
        except sp.CalledProcessError as e:
            typer.echo(f"An error occurred while running 'poetry remove {dependency}'.", color="red")
            sys.exit(1)



from enum import Enum
from typing import Dict, List, Optional
import click

from pydantic import BaseModel, EmailStr
import pathlib as pl

import typer

PROJECT_TEMPLATE_DEFAULT_RELATIVE_PATH = pl.Path(".")


class ProjectTypeEnum(str, Enum):
    FAST_API = "fastapi"
    POETRY_PKG = "poetry_pkg"
    PRIME_REACT = "prime-react"
    # GCP_CLOUD_FUNCTION = "gcp_cloud_function"
    # COOKIECUTTER = "cookiecutter"

    @staticmethod
    def get_monorepo_types() -> List["ProjectTypeEnum"]:
        return [ProjectTypeEnum.FAST_API, ProjectTypeEnum.POETRY_PKG]

    def __repr__(self) -> str:
        return self.value

    def __str__(self):
        return self.value


class ProjectTemplateOptions(BaseModel):
    name: str
    package_name: str
    description: str
    author_name: Optional[str] = "name surname"
    author_email: Optional[EmailStr] = "email@gmail.com"


    @classmethod
    def prompt(cls) -> "ProjectTemplateOptions":
        name = typer.prompt("Enter the project name", default="myproject")
        package_name = typer.prompt("Enter the package name for the project", default="myproject")
        description = typer.prompt("Enter a description for the project", default="A description for the project")
        author_name = typer.prompt("Enter the author name for the project", default="name surname")
        author_email = typer.prompt("Enter the author email for the project", default="email@gmail.com")

        return ProjectTemplateOptions(
            name=name,
            package_name=package_name,
            description=description,
            author_name=author_name,
            author_email=author_email
        )
        

class Dependency(BaseModel):
    name: str
    relative_path: Optional[pl.Path] = pl.Path(".")

    @classmethod
    def prompt(cls, available_dependencies: List["ProjectTemplate"]):
        names = [pt.project_options.name for pt in available_dependencies]
        name = typer.prompt("Enter the dependency name", type=click.Choice(names))

        selected_project_template = None
        for pt in available_dependencies:
            if pt.project_options.name == name:
                selected_project_template = pt
        
        if selected_project_template is None:
            raise ValueError("Invalid selected dependency..")
        

        # relative_path = typer.prompt("Enter the relative path of the dependency", default=pl.Path("."))

        return Dependency(
            name=name,
            relative_path=pl.Path("../") / selected_project_template.relative_path / selected_project_template.project_options.name
        )


class ProjectTemplate(BaseModel):
    project_type: ProjectTypeEnum
    relative_path: Optional[pl.Path] = PROJECT_TEMPLATE_DEFAULT_RELATIVE_PATH
    project_options: ProjectTemplateOptions
    dependencies: Optional[List[Dependency]] = []

    class Config:
        use_enum_values = True

    @property
    def extra_content(self) -> Dict:
        extra_content = self.project_options.model_dump(mode="json")

        extra_content["dependencies"] = {dependency.name: dependency.model_dump(mode="json") for dependency in self.dependencies}

        return extra_content
    
    @classmethod
    def prompt(cls, available_dependencies: List["ProjectTemplate"] = None) -> "ProjectTemplate":
        project_type = typer.prompt("Enter the project template type for the component you want to add", type=click.Choice(ProjectTypeEnum.get_monorepo_types()))

        relative_path = typer.prompt("Enter the relative path of the component you want to add", default=PROJECT_TEMPLATE_DEFAULT_RELATIVE_PATH)

        project_options = ProjectTemplateOptions.prompt()

        dependencies = []
        if available_dependencies:
            while typer.confirm("Do you want to add a dependency for this component? (y/n)", default=True):
                dependencies.append(Dependency.prompt(
                    available_dependencies=available_dependencies
                ))

        return ProjectTemplate(
            project_type=project_type,
            relative_path=relative_path,
            project_options=project_options,
            dependencies=dependencies
        )


if __name__ == "__main__":
    pt = ProjectTemplate.prompt()


import subprocess
from typing import List

import typer
from cookiecutter.main import cookiecutter
from tigr81 import LOCAL_REPO_LOCATION, REPO_LOCATION
from tigr81.cli_settings import CLI_SETTINGS
from tigr81.commands.scaffold.project_template import (
    ProjectTemplate,
    ProjectTemplateOptions,
    ProjectTypeEnum,
)
import pathlib as pl


def scaffold(
    project_type: ProjectTypeEnum,
    default: bool = False,
    output_dir: pl.Path = pl.Path("."),
):
    author_email = subprocess.run(
        ["git", "config", "user.email"], capture_output=True, text=True, check=True
    ).stdout.strip()
    author_name = author_email.split("@")[0]

    project_template = ProjectTemplate(
        project_type=project_type,
        project_options=ProjectTemplateOptions(
            name=project_type,
            package_name=project_type,
            description=project_type,
            author_name=author_name,
            author_email=author_email,
        ),
    )

    scaffold_project_template(project_template, default=default, output_dir=output_dir)


def scaffold_project_template(
    project_template: ProjectTemplate,
    default: bool = False,
    output_dir: pl.Path = pl.Path("."),
):
    __PROJECT_TEMPLATE_LOCATION = REPO_LOCATION
    checkout = "develop"

    if CLI_SETTINGS.tigr81_environment == "local":
        __PROJECT_TEMPLATE_LOCATION = LOCAL_REPO_LOCATION.as_posix()
        checkout = None

    typer.echo(
        f"Scaffolding a {project_template.project_type} project template from {__PROJECT_TEMPLATE_LOCATION}"
    )

    cookiecutter(
        template=__PROJECT_TEMPLATE_LOCATION,
        output_dir=output_dir,
        no_input=default,
        extra_context=project_template.extra_content,
        checkout=checkout,
        directory=f"project_templates/{project_template.project_type}",
    )


def scaffold_cookiecutter(
    project_type: ProjectTypeEnum, output_dir: pl.Path = pl.Path(".")
):
    __PROJECT_TEMPLATE_LOCATION = REPO_LOCATION
    checkout = "develop"

    if CLI_SETTINGS.tigr81_environment == "local":
        __PROJECT_TEMPLATE_LOCATION = LOCAL_REPO_LOCATION.as_posix()
    typer.echo(
        f"Scaffolding a {project_type} project template from {__PROJECT_TEMPLATE_LOCATION}"
    )

    cookiecutter(
        template=__PROJECT_TEMPLATE_LOCATION,
        output_dir=output_dir,
        no_input=False,
        checkout=checkout,
        directory=f"project_templates/{project_type}",
    )



def scaffold_monorepo(components: List[ProjectTemplate]):
    pass



import pathlib as pl
import shutil
from typing import Dict, List

import click
import typer
from pydantic import BaseModel

import tigr81.commands.core.scaffold as scaffold_core
from tigr81.commands.core.poetry_pm import PoetryPM
from tigr81.commands.monorepo.constants import MANIFEST_FILE_NAME
from tigr81.commands.monorepo.manifest import Manifest
from tigr81.commands.scaffold.project_template import ProjectTemplate
from tigr81.utils.read_yaml import read_yaml

app = typer.Typer()


@app.callback()
def callback():
    """
    Handle monorepo project
    """


@app.command()
def add():
    """Add a component to the monorepo project"""
    typer.echo("Adding a component to the monorepo project")

    manifest_dct = read_yaml(MANIFEST_FILE_NAME)
    manifest = Manifest(**manifest_dct)

    component = ProjectTemplate.prompt(available_dependencies=manifest.components)

    manifest.components.append(component)

    manifest.to_yaml(MANIFEST_FILE_NAME)

    typer.echo(
        "In order to scaffold also the component you need to run 'tigr81 monorepo scaffold'"
    )


@app.command()
def draw():
    """Generate the .png representation of the monorepo pkd dependencies"""
    manifest_dct = read_yaml(MANIFEST_FILE_NAME)
    manifest = Manifest(**manifest_dct)
    mf_digraph = manifest.to_graphviz_digraph()
    typer.echo("Rendering the manifest...")
    mf_digraph.render()


@app.command()
def clean():
    """Delete all resources related to the current monorepo"""
    typer.echo("Cleaning monorepo project")
    manifest_dct = read_yaml(MANIFEST_FILE_NAME)
    manifest = Manifest(**manifest_dct)

    confirmed = typer.confirm(
        f"Are you sure you want to delete all the resources related to the monorepo: {manifest.name}"
    )

    if not confirmed:
        typer.echo("Cleaning stopped..")
        return

    typer.echo(
        f"Deleting all resources related to monorepo related to the monorepo: {manifest.name}.."
    )
    if manifest.relative_path.exists():
        typer.echo("Deleting source code..")
        shutil.rmtree(manifest.relative_path)

    manifest_png = pl.Path(f"{manifest.name}.png")
    if manifest_png.exists():
        typer.echo(f"Deleting {manifest_png}..")
        manifest_png.unlink()

    manifest = pl.Path(manifest.name)

    if manifest.exists():
        typer.echo(f"Deleting {manifest}..")
        manifest.unlink()

    typer.echo(
        f"Deleted all resources related to monorepo related to the monorepo: {manifest.name}.."
    )


@app.command()
def init():
    """Initialize a monorepo project"""
    typer.echo("Inizializing a monorepo project")

    manifest_path = pl.Path(MANIFEST_FILE_NAME)

    if manifest_path.exists():
        overwrite = typer.confirm(
            "The manifest file already exists, do you want to overwrite?"
        )
        if not overwrite:
            typer.echo("Manifest file not overwritten!!")
            return

    manifest = Manifest.prompt()
    typer.echo("Creating manifest file...")
    manifest.to_yaml()


@app.command()
def install():
    """Install every component inside the monorepo project"""
    typer.echo("Installing all the components of the monorepo project")

    pc = PoetryPM()

    manifest_dct = read_yaml(MANIFEST_FILE_NAME)

    manifest = Manifest(**manifest_dct)

    for component in manifest.components:
        pc.install(
            cwd=manifest.relative_path
            / component.relative_path
            / component.project_options.name
        )


@app.command()
def remove():
    """Remove a component from the monorepo project"""
    manifest_dct = read_yaml(MANIFEST_FILE_NAME)

    manifest = Manifest(**manifest_dct)

    names = [pt.project_options.name for pt in manifest.components]
    name = typer.prompt(
        "Enter the name of the component you want to delete", type=click.Choice(names)
    )

    typer.echo(f"Removing the component {name} to the monorepo project")
    manifest.remove(component_name=name)

    mf_digraph = manifest.to_graphviz_digraph()
    typer.echo("Rendering the manifest...")
    mf_digraph.render()
    manifest.to_yaml()


@app.command()
def scaffold():
    """Scaffold a monorepo project"""
    # TODO: check if the file exists and add a method from_yaml in Manifest
    try:
        manifest_dct = read_yaml(MANIFEST_FILE_NAME)
    except FileNotFoundError:
        typer.echo(
            "Manifest not found.. make sure you are in the correct folder or if you want to create a new one run: tigr81 monorepo init"
        )
        raise typer.Exit()

    manifest = Manifest(**manifest_dct)

    typer.echo(
        f"Scaffolding a monorepo project from manifest located at {pl.Path.cwd() / MANIFEST_FILE_NAME}"
    )

    for component in manifest.components:
        component_location = (
            manifest.relative_path
            / component.relative_path
            / component.project_options.name
        )
        typer.echo(f"Scaffolding {component_location} component...")
        if not component_location.exists():
            scaffold_core.scaffold_project_template(
                project_template=component,
                default=True,
                output_dir=manifest.relative_path / component.relative_path,
            )
            typer.echo(f"Component {component_location} scaffolded successfully")
        else:
            typer.echo(f"Component {component.project_options.name} already exists..")

    typer.echo("Monorepo scaffolding completed successfully")

    mf_digraph = manifest.to_graphviz_digraph()

    typer.echo("Rendering the manifest...")
    mf_digraph.render()


# Define a function to interactively prompt the user for input for a Pydantic BaseModel
def prompt_for_model(model_type: BaseModel) -> BaseModel:
    input_data = {}
    for field_name, field_info in model_type.model_fields.items():
        if issubclass(field_info, BaseModel):
            input_data[field_name] = prompt_for_model(field_info)
        elif issubclass(field_info, List):
            inner_type = field_info.__args__[0]
            input_data[field_name] = []
            while True:
                item = prompt_for_model(inner_type)
                input_data[field_name].append(item)
                if typer.confirm("Add another item? (y/n)", default=True):
                    continue
                else:
                    break
        elif issubclass(field_info, Dict):
            key_type, value_type = field_info.__args__
            input_data[field_name] = {}
            while True:
                key = prompt_for_model(key_type)
                value = prompt_for_model(value_type)
                input_data[field_name][key] = value
                if typer.confirm("Add another item? (y/n)", default=True):
                    continue
                else:
                    break
        else:
            default_value = (
                field_info.default if hasattr(field_info, "default") else None
            )
            user_input = typer.prompt(
                f"Enter value for {field_name} (default is {default_value}): ",
                default=default_value,
            )
            input_data[field_name] = user_input
    return model_type(**input_data)
